/*


functions:
int get(void* key, void* value, void* error); 
int post(void* key, void* value, void* error); 
int delete(void* key, void* value, void* error); 


PA4 instructions:
This is a library, which will encode and decode your request and response messages.
For example, at the client side this library will encode your request message to the decided
message format, and then send it out to the server process. Similarly, it will decode the
response received from the server, and then hand out the decoded response to the KVClient
module.
Something complimentary will happen at the server end.
Status Codes for request message
GET: 1
PUT: 2
DEL: 3
Status Codes for response message
Success: 200
Error: 400 (with the appropriate error message)
Reasons for error could be GET key not found, DEL key not found etc.

https://moodle.iitb.ac.in/mod/forum/discuss.php?d=13716
The client library really consists of only 3 functions corresponding to the get, put and del. 
Internally, these messages manage encoding the request, sending the message, 
waiting for the reply and returning the appropriate value. 
So, a get API would like like int get(void* key, void* value, void* error); 
The return value is the status code. 
If the code is 200, then get will fill in the value with malloced memory. 
Else, it will malloc some memory into the error pointer and return it. And so on....

*/
#include "KVMessageFormat.h"
#include<stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h> 
#include <netdb.h> 
#include <netinet/in.h> 
#include <stdlib.h> 
#include <string.h> 
#include <unistd.h>
#include <sys/socket.h> 
#include <sys/types.h> 
#include <arpa/inet.h>

// to print a message
void printMessage(struct message *requestMessage)
{
    printf("[Message Generated]\n[[Status:%d]\n[Key:%s]\n[Value:%s]]\n",requestMessage->status,requestMessage->key,requestMessage->value);
    return ;
}
int  get(int sockfd,char* key,char *value,char *error)
{
    if(key==NULL)
    {
        printf("Invalid PARAM to get\n");
        return -1;
    }
    struct message *request=malloc(sizeof(struct message));
    request->status=1;//status code for get
    memcpy(request->key,key,256); //copied the complete key passed as parameter
    explicit_bzero(request->value,256*sizeof(char));//null value to to the request->value field
    struct message *reply=malloc(sizeof(struct message));
    write(sockfd, request, sizeof(struct message)); //passing to server and waiting for response
    read(sockfd,reply,sizeof(struct message));
    int status=reply->status;
    if(status==400)
    {
        memcpy(error,reply->key,256*sizeof(char)); //copying the error in the key field for an failed get

    }
    else
    {
        memcpy(value,reply->value,256*sizeof(char));
    }
    
    free(request);
    free(reply);
    return status; //return reply->key


}
int del(int sockfd,char *key,char *error)
{
    if(key==NULL)
    {
        printf("Invalid PARAM to del\n");
        return -1;
    }
    struct message *request=malloc(sizeof(struct message));
    request->status=2;//status code for delete
    memcpy(request->key,key,256); //copied the complete key passed as parameter
    explicit_bzero(request->value,256*sizeof(char));//null value to to the request->value field

    struct message *reply=malloc(sizeof(struct message));
    write(sockfd, request, sizeof(struct message)); //passing to server and waiting for response to the del request
    read(sockfd,reply,sizeof(struct message));
    int status=(int)reply->status;
    if(status==400)
    {
        memcpy(error,reply->key,256*sizeof(char)); //copying the error in the key field for an failed del
    }
    free(request);
    free(reply);
    return status; //return reply->key

}
int put(int sockfd,char *key,char *value,char *error)
{
    if(key==NULL)
    {
        printf("Invalid PARAM to del\n");
        return -1;
    }
    struct message *request=malloc(sizeof(struct message));
    request->status=3;//status code for delete
    memcpy(request->key,key,256); //copied the complete key passed as parameter
    memcpy(request->value,value,256); //copied the complete key passed as parameter
    struct message *reply=malloc(sizeof(struct message));
    write(sockfd, request, sizeof(struct message)); //passing to server and waiting for response to the del request
    read(sockfd,reply,sizeof(struct message));
    int status=reply->status;
    if(status==400)
    {
        memcpy(error,reply->key,256*sizeof(char)); //copying the error in the key field for an failed del
    }
    else
    {
        printMessage(reply);
    }
    
    free(request);
    free(reply);
    return status; //return reply->key

}

